-- synthesis script for bottling process

-- configure
faudes.StateNamesOff()

--------------------------------------------------------------
--------------------------------------------------------------
-- CreateQueue
--------------------------------------------------------------
--------------------------------------------------------------

-- build automaton to represent a fifo-buffer 
--
-- pstr: prefix of put-event labels
-- tstr: prefix of take-event label
-- estr: error-event label (empty string for no error)
-- m: number of symbols to put/take (max. 9, i.e., one digit)
-- n: depth of buffer
--
-- example: gen=CreateQueue("P","D","I",3,2) for a depth two
-- buffer of product requests P1,P2,P3 and respective processes 
-- D1,D2,D3. On buffer overflow, any further P event is effectively
-- ignored, except formal acknowledged by the error indicator I


function CreateQueue(pstr,tstr,estr,m,n)
  
  -- repare result
  local gRes=faudes.Generator()
  if estr~="" then gRes:InsEvent(estr) end

  -- have initial state with empty fifo in todo list
  local todo={"s"}
  gRes:InsInitState("s")
  gRes:SetMarkedState("s")

  -- work on todo list
  while # todo > 0 do
    -- get state name, aka encoded fifo state
    local x1str = todo[1]
    table.remove(todo,1)
    -- append any put-events
    for j=1,m do
      -- case 1: can put
      if string.len(x1str)-1<n then
        local x2str = x1str .. j
        table.insert(todo,x2str)
        gRes:InsState(x2str)
        local evstr = pstr .. j
        gRes:InsEvent(evstr)
        gRes:SetTransition(x1str, evstr, x2str)        
      end
      -- case 2: overflow error
      if string.len(x1str)-1==n and estr~="" then
        local x2str = x1str .. "e"
        if not gRes:ExistsState(x2str) then gRes:InsState(x2str) end
        local evstr = pstr .. j
        gRes:InsEvent(evstr)
        gRes:SetTransition(x1str, evstr, x2str)        
        gRes:SetTransition(x2str, estr, x1str)        
      end
      -- case 3: can take
      if string.len(x1str)>1 then
        local x2str = "s" .. string.sub(x1str,3)  
        local evstr= tstr .. string.sub(x1str,2,2)
        gRes:InsEvent(evstr)
        gRes:SetTransition(x1str, evstr, x2str)        
      end
    end
  end

  -- pass on result
  return gRes:Copy()
end


--------------------------------------------------------------
--------------------------------------------------------------
-- Test abstractions for consistency  (experimental)
--
-- See "Interner Bericht Nr. 7 (2nd ed)" and susequent LRT reports
-- for a detailed discussion.
--
-- This implementation is meant for development and experiments,
-- it is not optimized for efficiency.
--------------------------------------------------------------
--------------------------------------------------------------

function faudes.IsConsistentAbstraction(gL,sCtrl,sObs)

-- Parameters
-- gL              plant model to test
-- sCtrl           controllable events
-- sObs            observable events
--
-- gL must be determinitsic and trim
-- alphabets muts be subsets of the alphabet from gL

  -- report
  faudes.Print(string.format("IsConsistentAbstraction(): #states=%d",gL:Size()))

  -- debug with consistent state indices
  gL:MinStateIndex()

  -- alphabets
  local sUCtrl = gL:Alphabet()-sCtrl
  local sUObs =  gL:Alphabet()-sObs

  -- projected loop
  local gLo=faudes.Generator()
  faudes.Project(gL,sObs,gLo)

  -- report
  faudes.Print(string.format("IsConsistentAbstraction(): #projection-states=%d",gLo:Size()))

  -- extended realisation incl. observer states
  local gLLo=faudes.Generator()
  local cmap=faudes.ProductCompositionMap()
  faudes.Parallel(gL,gLo,cmap,gLLo) 

  -- report
  faudes.Print(string.format("IsConsistentAbstraction(): #observer-states=%d",gLLo:Size()))

  -- debug files
  --[[
  gLLo:Write("tmp_labsdbg_llo.gen")
  gLo:Write("tmp_labsdbg_lo.gen")
  --]]

  -- ensure trim generators
  -- (not needed?)
  if not gLLo:IsTrim() then
    gLLo:Trim()
  end

  -- one-step target control operator (tests A,B and C)
  local function omegaABC(xM) 
    faudes.Print(string.format("IsConsistentAbstraction:omegaABC(): #target=%d",xM:Size()))
    local xRes=faudes.IndexSet()
    local resa=0
    local resb=0
    local resc=0
    local sit = gLLo:StatesBegin()
    while sit ~= gLLo:StatesEnd() do
      local pass=false
      local enevs= gLLo:ActiveEventSet(sit:Index())
      local uctrlenevs = enevs * sUCtrl
      local uobsenevs = enevs * sUObs
      local oenevs= enevs * sObs
      -- case A: unobservable event to target enabled
      if (not pass) and not uobsenevs:Empty() then
        local tit = gLLo:TransRelBegin(sit:Index())
        while tit~= gLLo:TransRelEnd(sit:Index()) do
          if xM:Exists(tit:X2()) and sUObs:Exists(tit:Ev()) then
            --print(sit:Index(), "pass case A:",enevs:ToString())
            xRes:Insert(sit:Index())
            resa=resa+1
            pass=true
            break
          end
        tit:Inc()
        end
      end     
      -- case B: uncontrollable event to target enabled
      if (not pass) and not uctrlenevs:Empty() then
        local tit = gLLo:TransRelBegin(sit:Index())
        while tit~= gLLo:TransRelEnd(sit:Index()) do
          if xM:Exists(tit:X2()) and sUCtrl:Exists(tit:Ev()) then
            --print(sit:Index(), "pass case B:",enevs:ToString())
            xRes:Insert(sit:Index())
            resb=resb+1
            pass=true
            break
          end
        tit:Inc()
        end
      end     
      -- case C: find consistent controllable and observable transitions to target
      if (not pass) and not oenevs:Empty() then
        local q=cmap:Arg1State(sit:Index())
        local z=cmap:Arg2State(sit:Index())
        local zenevs = gLo:ActiveEventSet(z)
        if zenevs <= sCtrl then
          pass=true
          local eit= zenevs:Begin()
          while eit ~= zenevs:End() do
            if not xM:Exists(gLLo:SuccessorState(sit:Index(),eit:Index())) then 
              pass=false
              break
            end
            eit:Inc()
          end
          if pass then
            --print(sit:Index(), "pass case C:",enevs:ToString())
            xRes:Insert(sit:Index())
            resc=resc+1
          end
        end
      end
      -- all cases failed
      if not pass then
        local enevs=gLLo:ActiveEventSet(sit:Index())
        local z=cmap:Arg2State(sit:Index())
        local zenevs = gLo:ActiveEventSet(z)
        --print(sit:Index(), "unresolved: ",enevs,zenevs)
      end
      -- iterate all states
      sit:Inc()
    end
    faudes.Print(string.format("IsConsistentAbstraction:omegaABC(): #resolved_A=%d",resa))
    faudes.Print(string.format("IsConsistentAbstraction:omegaABC(): #resolved_B=%d",resb))
    faudes.Print(string.format("IsConsistentAbstraction:omegaABC(): #resolved_C=%d",resc))
    faudes.Print(string.format("IsConsistentAbstraction:omegaABC(): #resolved_+=%d",xRes:Size()))
    return xRes:Copy()
  end


  -- star-step target control operator (test D)
  local function omegaD(xM) 
    faudes.Print(string.format("IsConsistentAbstraction:omegaD(): #target=%d",xM:Size()))
    local xRes=faudes.IndexSet()
    local sit = gLLo:StatesBegin()
    while sit ~= gLLo:StatesEnd() do
      local pass=false
      -- test for existence of transition to target
      local tit = gLLo:TransRelBegin(sit:Index())
      while tit~= gLLo:TransRelEnd(sit:Index()) do
        if xM:Exists(tit:X2()) then
          pass=true
          break
        end
        tit:Inc()
      end
      -- test for not allready in target
      if xM:Exists(sit:Index()) then
        pass=false
      end
      -- candidats not in target but with transition to target
      if pass then
        faudes.Print(string.format("IsConsistentAbstraction:omegaD(): testing state %d",sit:Index()))
        -- compute reachable qz-states outside the target
        local xReachQZ=faudes.IndexSet()
        xReachQZ:Insert(sit:Index())
        local todo=xReachQZ:Copy()
        while not todo:Empty() do
          --print("xReachQZ: todo", todo)
          local ins=faudes.IndexSet()
          local xit = todo:Begin()
          while xit~= todo:End() do
            local tit = gLLo:TransRelBegin(xit:Index())
            while tit~= gLLo:TransRelEnd(xit:Index()) do
              if (not xM:Exists(tit:X2())) and (not xReachQZ:Exists(tit:X2())) then
                ins:Insert(tit:X2())
              end
              tit:Inc()
            end
            xit:Inc()
          end
          xReachQZ:InsertSet(ins)
          todo=ins:Copy()
        end
        --xReachQZ:Write()
        -- compute reachable z-states with no corresponing qz-states in target
        local xReachZ=faudes.IndexSet()
        xReachZ:Insert(cmap:Arg2State(sit:Index()))
        local todo=xReachZ:Copy()
        while not todo:Empty() do
          --print("xReachZ: todo", todo)
          local ins=faudes.IndexSet()
          local zit = todo:Begin()
          while zit~= todo:End() do
            local tit = gLo:TransRelBegin(zit:Index())
            while tit~= gLo:TransRelEnd(zit:Index()) do
              if not xReachZ:Exists(tit:X2()) then
                local safe=true
                local xit = gLLo:StatesBegin()
                while xit~= gLLo:StatesEnd() do
                  if not xReachQZ:Exists(cmap:Arg2State(xit:Index())) then
                     safe=false
                     break
                  end
                  xit:Inc()
                end
                if safe then
                  ins:Insert(tit:X2())
                end
              end
              tit:Inc()
            end
            zit:Inc()
          end
          xReachZ:InsertSet(ins)
          todo=ins:Copy()
        end
        --xReachZ:Write()
        -- test whether z-component reaches marking
        pass = (xReachZ * gLo:MarkedStates()):Empty() 
      end
      -- record pass
      if pass then
        xRes:Insert(sit:Index())
      end
      -- iterate all states
      sit:Inc()
    end
    xRes:InsertSet(xM)
    faudes.Print(string.format("IsConsistentAbstraction:omegaD(): #resolved=%d",xRes:Size()))
    return xRes:Copy()
  end


  -- alternative star-step target control operator (undocumented test E, implementation using SCCs)
  local function omegaE(xM) 
    faudes.Print(string.format("IsConsistentAbstraction:omegaE(): #target=%d",xM:Size()))
    local xRes=faudes.IndexSet()
    -- iterate over relevant sccs
    local sfilter = faudes.SccFilter()
    sfilter:IgnoreTrivial(true)
    sfilter:StatesAvoid(xM)
    local scc=faudes.IndexSet()
    while faudes.ComputeNextScc(gLLo,sfilter,scc) do
      print("scc: found with #",scc:Size())
      local pass=true
      -- map scc to gLo
      local scc0=faudes.IndexSet()
      local sit=scc:Begin()
      while sit~=scc:End() do
        scc0:Insert(cmap:Arg2State(sit:Index()))
        sit:Inc()
      end
      -- test scc0 to be unmarked
      if not (scc0 * gLo:MarkedStates()):Empty() then
        print("scc: discard for high-level marking")
        pass=false
      end
      -- test scc exits to reach target
      local sit=scc:Begin()
      while pass and sit~=scc:End() do
        local tit=gLLo:TransRelBegin(sit:Index())
        while pass and tit~=gLLo:TransRelEnd(sit:Index()) do
          local qnxt=tit:X2()
          if (not scc:Exists(qnxt)) and (not xM:Exists(qnxt)) then 
            pass=false
            print("scc: failed for non-target exit")
          end
          tit:Inc()
        end
        sit:Inc()
      end
      -- test scc exits are controllable and observabe
      --[[
      local sit=scc:Begin()
      while pass and sit~=scc:End() do
        local tit=gLLo:TransRelBegin(sit:Index())
        while pass and tit~=gLLo:TransRelEnd(sit:Index()) do
          if not scc:Exists(tit:X2()) then
            if not sCtrl:Exists(tit:Ev()) then 
              pass=false
              print("scc: failed for non-ctrl exit at", tit:X1())
            end
            if not sObs:Exists(tit:Ev()) then 
              pass=false
              print("scc: failed for non-obs exit at", tit:X1())
            end
          end
          tit:Inc()
        end
        sit:Inc()
      end
      --]]
      -- build scc0 automata
      if pass then
        print("scc: build scc0 automaton")
        gscc0=faudes.Generator()
        local sit=scc0:Begin()
        while pass and sit~=scc0:End() do
          local tit=gLo:TransRelBegin(sit:Index())
          while pass and tit~=gLo:TransRelEnd(sit:Index()) do
            gscc0:InsState(tit:X1())
            gscc0:InsEvent(tit:Ev())
            gscc0:InsState(tit:X2())     
            gscc0:SetTransition(tit:X1(),tit:Ev(),tit:X2())
            tit:Inc()
          end
          sit:Inc()
        end
      end
      -- build scc automata
      if pass then
        print("scc: build scc automaton")
        gscc=faudes.Generator()
        local init=false
        local sit=scc:Begin()
        while sit~=scc:End() do
          local tit=gLLo:TransRelBegin(sit:Index())
          while pass and tit~=gLLo:TransRelEnd(sit:Index()) do
            gscc:InsState(tit:X1())
            gscc:InsEvent(tit:Ev())
            gscc:InsState(tit:X2())     
            gscc:SetTransition(tit:X1(),tit:Ev(),tit:X2())
            if xM:Exists(tit:X2()) then 
              gscc0:InsMarkedState(cmap:Arg2State(tit:X2()))
              gscc:InsMarkedState(tit:X2())
              if not init then
                gscc:InsInitState(tit:X1())
                gscc0:InsInitState(cmap:Arg2State(tit:X1()))
                init=true
              end
            end
            tit:Inc()
          end
          sit:Inc()
        end
        --gscc:Write()
        --gscc0:Write()
      end
      -- project scc
      if pass then
        gsccp0=faudes.Generator()
        faudes.Project(gscc,sObs,gsccp0)
        if not faudes.LanguageInclusion(gsccp0,gscc0) then
          faudes.Print("IsConsistentAbstraction:omegastr(): assertion failed: internal ERROR")
        end
      end
      -- test p0 Lm(gscc) == Lm(gscc0)
      if pass then
        if not faudes.LanguageEquality(gsccp0,gscc0) then
          print("scc: failed for unimplemented high-level exit strings")
          pass=false
        end
      end
      -- test natural observer property
      --[[
      if pass then
        if not faudes.IsNaturalObserver(gscc,sObs) then
          print("scc: failed for not being observed by po")
          pass=false
        end
      end
      --]]
      -- record pass
      if pass then
        print("scc: pass")
        --xRes:InsertSet(scc)
        xRes:InsertSet(gscc:InitStates())
      end  
    -- iterate sccs
    end

    -- report
    faudes.Print(string.format("IsConsistentAbstraction:omegaE(): #resolved_*=%d",xRes:Size()))
    return xRes:Copy()
  end

  -- test locally complete control
  local xQZ = gLLo:States():Copy()
  local xOmegaQZ = omegaABC(xQZ)

  -- report unresolved
  if xQZ ~=xOmegaQZ then
    faudes.Print("IsConsistentAbstraction: failed test for locally complete control")
    local sit = gLLo:StatesBegin()
    while sit ~= gLLo:StatesEnd() do
      if not xOmegaQZ:Exists(sit:Index()) then
        local enevs=gLLo:ActiveEventSet(sit:Index())
        local z=cmap:Arg2State(sit:Index())
        local zenevs = gLo:ActiveEventSet(z)
        faudes.Print(string.format("unresolved state %d with enabled events %s",sit:Index(),enevs:ToString()))
        faudes.Print(string.format("  projection enabled events %s",zenevs:ToString()))
      end
      sit:Inc()
    end
    return false
  end
  faudes.Print("IsConsistentAbstraction: passed test for locally complete control")

  -- strict test for non-blocking control (as in "Interner Bericht Nr. 7 (1st ed.)")
  local xMi = gLLo:MarkedStates()
  local sz=0
  while xMi:Size() > sz do
    sz=xMi:Size()
    xMi = omegaABC(xMi) + xMi
  end
  if xQZ==xMi then
    faudes.Print("IsConsistentAbstraction: passed test for nonblocking control using operator omegaABC")
  else
    faudes.Print("IsConsistentAbstraction: failed test for nonblocking control using operator omegaABC")
  end

  -- relaxed test for non-blocking control (as in "Interner Bericht Nr. 7 (2nd ed.)", incl omegaD)
  if xQZ~=xMi then
    while true do
      sz=xMi:Size()
      xMi = omegaD(xMi)  
      if xMi:Size()==sz then break end
      while xMi:Size() > sz do
        sz=xMi:Size()
        xMi = omegaABC(xMi) + xMi
      end
      if xMi:Size()==xQZ:Size() then break end
    end
    if xMi:Size()==xQZ:Size() then
      faudes.Print("IsConsistentAbstraction: passed test for nonblocking control, iterating omegaABC and omegaD")
    end
  end

  -- report unresolved
  if xQZ~=xMi then
    faudes.Print("IsConsistentAbstraction: failed both tests for nonblocking control")
    local sit = gLLo:StatesBegin()
    while sit ~= gLLo:StatesEnd() do
      if not xMi:Exists(sit:Index()) then
        local enevs=gLLo:ActiveEventSet(sit:Index())
        local z=cmap:Arg2State(sit:Index())
        local q=cmap:Arg1State(sit:Index())
        local zenevs = gLo:ActiveEventSet(z)
        local tevs=faudes.EventSet()
        local eit= enevs:Begin()
        while eit ~= enevs:End() do
          if xMi:Exists(gLLo:SuccessorState(sit:Index(),eit:Index())) then 
             tevs:Insert(eit:Index())
          end
          eit:Inc()
        end
        if not tevs:Empty() then
          faudes.Print(string.format("unresolved state %d with enabled events %s",sit:Index(),enevs:ToString()))
          faudes.Print(string.format("  plant state id %d",q))
          faudes.Print(string.format("  projection enabled events %s",zenevs:ToString()))
          faudes.Print(string.format("  target enabled events %s",tevs:ToString()))
        end
      end
      sit:Inc()
    end
    return false
  end

  faudes.Print("IsConsistentAbstraction: all relevant tests passed")
  return true
end

-- abstraction demo on test data
print("=== abstraction demo 1: consistency tests")
ldemo = faudes.System("labsdemo1.gen")
sigmahi = ldemo:ObservableEvents()
sigmac = ldemo:ControllableEvents()
absok=faudes.IsConsistentAbstraction(ldemo,sigmac,sigmahi)
if absok then
  print("abstraction demo 1: consistency test passed (expected)")
else
  print("abstraction demo 1: consistency test FAILED (test case ERROR)")
end
obsok=faudes.IsNaturalObserver(ldemo,sigmahi)
if obsok then
  print("abstraction demo 1: natural observer test passed (test case ERROR)")
else
  print("abstraction demo 1: natural observer test failed (expected)")
end

-- abstraction demo on test data
print("=== abstraction demo 2: consistency tests")
ldemo = faudes.System("labsdemo2.gen")
sigmahi = ldemo:ObservableEvents()
sigmac = ldemo:ControllableEvents()
absok=faudes.IsConsistentAbstraction(ldemo,sigmac,sigmahi)
if absok then
  print("abstraction demo 2: consistency test passed (expected)")
else
  print("abstraction demo 2: consistency test FAILED (test case ERROR)")
end
obsok=faudes.IsNaturalObserver(ldemo,sigmahi)
if obsok then
  print("abstraction demo 2: natural observer test passed (test case ERROR)")
else
  print("abstraction demo 2: natural observer test failed (expected)")
end




-----------------------------------
-----------------------------------
-- transport plant incl process
-----------------------------------
-----------------------------------

-- report
print("=== plant models")

-- alternative transport models (see FtcTransport)
ltransf=faudes.System("htransport.gen");
ltransa=faudes.System("htransabs.gen");
ltranss=faudes.System("ltrans.gen");

-- choose
--print("=== using full transport model")
--lplant=ltransf
print("=== using abstract transport model")
lplant=ltransa
--print("=== using simple transport model")
--lplant=ltranss

-- merge with process model
lproc=faudes.System("lproc.gen")
faudes.Parallel(lplant,lproc,lplant)

-- report
lplant:Name("lplant")
lplant:SWrite()


-----------------------------------
-----------------------------------
-- synthesis for process
-----------------------------------
-----------------------------------

-- report
print("=== synthesis: process")

-- load
eproc=faudes.Generator("eproc.gen")

-- synthesis
sigma= lplant:Alphabet() + eproc:Alphabet()
sigmac = lplant:ControllableEvents() + (sigma - lplant:Alphabet())
sigmac:Name("sigmac")
sigmac:Write()
faudes.InvProject(lplant,sigma)
faudes.InvProject(eproc,sigma)
lplant:SetControllable(sigmac)
kprocess=faudes.System()
faudes.SupConNB(lplant,eproc,kprocess)
faudes.StateMin(kprocess)
kprocess:Name("kprocess")

-- report
kprocess:SWrite()
kprocess:Write("tmp_kprocess.gen")

-- abstract with natural observer
-- (expected high-level alphabet for nat. obs.: S1 S3 V R D1 D2 D3)
print("=== abstraction: test for natural observer")
sigmahi=faudes.Alphabet();
sigmahi:FromString('<A> S1 S3 V R D1 D2 D3 </A>')
obsok=faudes.IsNaturalObserver(kprocess,sigmahi)
if obsok then
  print("=== first guess passed natural observer test (expected)")
else
  print("=== first guess failed natural observer test (ERROR)")
end
obsok=faudes.IsConsistentAbstraction(kprocess,sigmac,sigmahi)
if obsok then
  print("=== first guess passed consistency test (expected)")
else
  print("=== first guess failed consistency test (ERROR)")
end
--[[
print("=== abstraction: find natural observer")
sigmahi=faudes.Alphabet();
sigmahi:FromString('<A> S1 S3 V R D1 D2 D3 </A>')
faudes.NaturalObserverExtension(kprocess,sigmahi)
--]]
sigmahi:Name("sigmahi")
sigmahi:Write()
hprocess=faudes.System()
faudes.Project(kprocess,sigmahi,hprocess)
hprocess:Name("hprocess")
hprocess:SWrite()
hprocess:Write("tmp_hprocess.gen")


-----------------------------------
-----------------------------------
-- synthesis for acknowledgement
-----------------------------------
-----------------------------------

-- report
print("=== synthesis: acknowledge")

-- generate buffer
eackbuf=CreateQueue("D","p","",3,4)
eackbuf:Write("tmp_eackbuf.gen")
eackbuf:SWrite()
eexit=faudes.Generator("eexit.gen")
eackspec=faudes.Generator()
faudes.Parallel(eexit,eackbuf,eackspec)

-- synthesis
sigma= hprocess:Alphabet() + eackspec:Alphabet()
sigmac = hprocess:ControllableEvents() + (sigma - hprocess:Alphabet())
sigmac:Name("sigmac")
sigmac:Write()
faudes.InvProject(hprocess,sigma)
faudes.InvProject(eackspec,sigma)
hprocess:SetControllable(sigmac)
kacknow=faudes.System()
faudes.SupConNB(hprocess,eackspec,kacknow)
kacknow:Name("kacknow")

-- report
kacknow:SWrite()
kacknow:Write("tmp_kacknow.gen")

-- abstract
-- (construct observer alphabet)
print("=== abstraction: construct natural observer")
sigmahi=faudes.Alphabet();
sigmahi:FromString('<A> D1 D2 D3 </A>')
faudes.NaturalObserverExtension(kacknow,sigmahi)
sigmahi:Name("sigmahi")
sigmahi:Write()

-- abstract with control consistent observer
-- (guess and verify observer alphabet)
print("=== abstraction: using consistency test")
sigmahi=faudes.Alphabet();
sigmahi:FromString('<A> D1 D2 D3 </A>')
sigmahi:Name("sigmahi")
sigmahi:Write()
print("=== abstraction: test consistency")
obsok=faudes.IsConsistentAbstraction(kacknow,sigmac,sigmahi)
if obsok then
  print("abstraction: consistency test: passed")
else
  print("abstraction: consistency test: FAILED")
end
hacknow=faudes.System(kacknow)
faudes.Project(kacknow,sigmahi,hacknow)
hacknow:Name("hacknow")
hacknow:SWrite()
hacknow:Write("tmp_hacknow.gen")

-----------------------------------
-----------------------------------
-- synthesis for requests
-----------------------------------
-----------------------------------

-- report
print("=== synthesis: requests")

-- generate buffer
ereqbuf=CreateQueue("P","D","I",3,4)
ereqbuf:SWrite()
ereqbuf:Write("tmp_ereqbuf.gen")

-- merge plant with random request model
-- lreq=faudes.System("lreq.gen")
-- faudes.Parallel(hacknow,lreq,hacknow)

-- synthesis
sigma= hacknow:Alphabet() + ereqbuf:Alphabet()
sigma:Name("sigma")
sigma:Write()
sigmac = hacknow:ControllableEvents() + (sigma - hacknow:Alphabet())
sigmac:Name("sigmac")
sigmac:Write()
faudes.InvProject(hacknow,sigma)
faudes.InvProject(ereqbuf,sigma)
hacknow:SetControllable(sigmac)
krequest=faudes.System()
faudes.SupConNB(hacknow,ereqbuf,krequest)
krequest:Name("krequest")

-- report
krequest:SWrite()
krequest:Write("tmp_krequest.gen")


